@@ -1,6 +1,5 @@
 {-# LANGUAGE TypeFamilies #-}
 {-# LANGUAGE FlexibleInstances #-}
-{-# LANGUAGE FunctionalDependencies #-}
 {-# LANGUAGE MultiParamTypeClasses #-}
 {-# LANGUAGE CPP #-}
 {-# LANGUAGE OverloadedStrings #-}
@@ -88,11 +87,11 @@ import qualified Data.Text.IO as TIO
 import Data.Bits
 import Data.ByteString.Builder (toLazyByteString, char7, byteString, hPutBuilder)
 import Data.ByteString.Builder.Extra (flush)
-import           GHC.IO.Handle.Internals         (wantWritableHandle)
 import           GHC.IO.Encoding.Types           (textEncodingName)
-import           GHC.IO.Handle.Types             (Handle__ (..))
 import qualified Data.ByteString as B
-import           System.IO                  (localeEncoding)
+import           System.IO                  (hGetEncoding, localeEncoding)
+import           System.IO.Error            (illegalOperationErrorType,
+                                             mkIOError, ioError)
 import           GHC.Foreign                (peekCString, withCString)
 import Data.Semigroup (Semigroup (..))
 
@@ -302,9 +301,14 @@ logStickyDone = logOther "sticky-done"
 -- that encompasses the logSticky->logStickyDone pairing.
 
 canUseUtf8 :: MonadIO m => Handle -> m Bool
-canUseUtf8 h = liftIO $ wantWritableHandle "canUseUtf8" h $ \h_ -> do
-  -- TODO also handle haOutputNL for CRLF
-  return $ (textEncodingName <$> haCodec h_) == Just "UTF-8"
+canUseUtf8 h = liftIO $ do
+  isWritable <- hIsWritable h
+  unless isWritable $ ioError $ mkIOError illegalOperationErrorType
+                                          "canUseUtf8"
+                                          (Just h)
+                                          Nothing
+  maybeEnc <- hGetEncoding h
+  return $ fmap textEncodingName maybeEnc == Just "UTF-8"
 
 -- | Create a 'LogOptions' value which will store its data in
 -- memory. This is primarily intended for testing purposes. This will
